<?php

namespace Drupal\html_segmenter;

/**
 * Interface for the HtmlSegmenter service.
 */
interface HtmlSegmenterInterface {

  /**
   * Extracts the translatable parts of HTML values.
   *
   * @param array $values
   *   An array of HTML strings.
   *
   * @return string[]
   *   A flat array of translatable text parts.
   */
  public function extractTranslatableHtmlValues(array $values): array;

  /**
   * Merges raw translations into original HTML strings.
   *
   * @param string[] $original
   *   Original HTML values.
   * @param string[] $extracted
   *   Extracted translatable parts.
   * @param string[] $translations
   *   Translated versions of the extracted parts.
   *
   * @return string[]
   *   HTML strings with translated parts reinserted.
   */
  public function mergeTranslatedHtmlValues(array $original, array $extracted, array $translations): array;
}
