<?php

namespace Drupal\Tests\html_segmenter\Unit;

use Drupal\html_segmenter\HtmlSegmenter;
use PHPUnit\Framework\TestCase;

class HtmlSegmenterTest extends TestCase {

  protected HtmlSegmenter $segmenter;

  protected function setUp(): void {
    parent::setUp();
    $this->segmenter = new HtmlSegmenter();
  }

  public function testExtractTranslatableHtmlValuesSimple() {
    $input = ['Hello <b>world</b>!'];
    $expected = ['Hello ', '!', 'world'];
    $result = $this->segmenter->extractTranslatableHtmlValues($input);
    $this->assertEquals($expected, $result);
  }

  public function testExtractTranslatableHtmlValuesWithToken() {
    $input = ['Welcome, @username!'];
    $expected = ['Welcome, ', '!'];
    $result = $this->segmenter->extractTranslatableHtmlValues($input);
    $this->assertEquals($expected, $result);
  }

  public function testExtractTranslatableHtmlValuesWithMultipleTokens() {
    $input = ['Hello, @user! Your order #{{ order_id }} is ready.'];
    $expected = ['Hello, ', '! Your order #', ' is ready.'];
    $result = $this->segmenter->extractTranslatableHtmlValues($input);
    $this->assertEquals($expected, $result);
  }

  public function testExtractTranslatableHtmlValuesWithNestedHtmlTags() {
    $input = ['<div>Hello <b>dear <i>world</i></b>!</div>'];
    // Extraction order may be: ['Hello ', '!', 'dear ', 'world']
    $expected = ['Hello ', '!', 'dear ', 'world'];
    $result = $this->segmenter->extractTranslatableHtmlValues($input);
    $this->assertEquals($expected, $result);
  }

public function testExtractTranslatableHtmlValuesWithTokenInHtmlTag() {
    $input = ['Hello <b>[custom:token-machine-name]</b>!'];
    // The token should be excluded, so only 'Hello ' and '!' are translatable.
    $expected = ['Hello ', '!'];
    $result = $this->segmenter->extractTranslatableHtmlValues($input);
    $this->assertEquals($expected, $result);
  }

  public function testMergeTranslatedHtmlValues() {
    $original = ['Hello <b>@name</b>!'];
    $extracted = ['Hello ', '!'];
    $translations = ['Buongiorno ', '!'];
    $expected = ['Buongiorno <b>@name</b>!'];
    $result = $this->segmenter->mergeTranslatedHtmlValues($original, $extracted, $translations);
    $this->assertEquals($expected, $result);
  }

  public function testMergeTranslatedHtmlValuesWithToken() {
    $original = ['Welcome, @username!'];
    $extracted = ['Welcome, ', '!'];
    $translations = ['Benvenuto, ', '!'];
    $expected = ['Benvenuto, @username!'];
    $result = $this->segmenter->mergeTranslatedHtmlValues($original, $extracted, $translations);
    $this->assertEquals($expected, $result);
  }

  public function testMergeTranslatedHtmlValuesSimpleExtractionOrder() {
    $original = ['Hello <b>world</b>!'];
    // Extraction order is: ['Hello ', '!', 'world']
    $extracted = ['Hello ', '!', 'world'];
    $translations = ['Ciao ', '!', 'mondo'];
    $expected = ['Ciao <b>mondo</b>!'];
    $result = $this->segmenter->mergeTranslatedHtmlValues($original, $extracted, $translations);
    $this->assertEquals($expected, $result);
  }

  public function testMergeTranslatedHtmlValuesWithNestedHtmlTags() {
    $original = ['<div>Hello <b>dear <i>world</i></b>!</div>'];
    $extracted = ['Hello ', '!', 'dear ', 'world'];
    $translations = ['Ciao ', '!', 'caro ', 'mondo'];
    $expected = ['<div>Ciao <b>caro <i>mondo</i></b>!</div>'];
    $result = $this->segmenter->mergeTranslatedHtmlValues($original, $extracted, $translations);
    $this->assertEquals($expected, $result);
  }

  public function testMergeTranslatedHtmlValuesWithTokenInHtmlTag() {
    $original = ['Hello <b>[custom:token-machine-name]</b>!'];
    $extracted = ['Hello ', '!'];
    $translations = ['Ciao ', '!'];
    $expected = ['Ciao <b>[custom:token-machine-name]</b>!'];
    $result = $this->segmenter->mergeTranslatedHtmlValues($original, $extracted, $translations);
    $this->assertEquals($expected, $result);
  }

}